from sympy import symbols, sympify, Eq
import sympy as sp
from math import floor, log10
from copy import deepcopy


def solve(info):
    inputs = info["input"]
    formula = info["formula"]
    solve_method = info["solve_method"]
    bonus_method = info["Bonus"]

    unitConvert(inputs)
    output_dict = deepcopy(inputs)
    extra = bonus_method(info)

    if formula != "":
        sol = solveEquation(formula, inputs)
        list_to_5_sig_figs(sol)
    else:
        info["output"] = extra


    for i in inputs:
        if safe_float(inputs[i][0], ""):
            output_dict[i][0] = safe_float(inputs[i][0], "")
        else:
            try:
                output_dict[i][0] = sol
            except Exception:
                pass

    info["output"] = output_dict

    return info


def safe_float(value, special_type):
    try:
        result = float(value)
        return result
    except (ValueError, TypeError):
        result = None


def unitConvert(inputs):
    for i, element in enumerate(inputs):
        # print(f"Units.{inputs[element][2]}")
        module = __import__(f"Units.{inputs[element][2]}", fromlist=['*'])

        class_obj = None
        class_count = 0
        for name, obj in module.__dict__.items():
            if isinstance(obj, type):
                class_count += 1
                if class_count == 2:
                    class_obj = obj
                    break

        my_obj = class_obj()

        unit_dict = my_obj.giveDict()

        # print(f"{unit_dict[inputs[element][3]]=}")
        # print(f"{inputs[element]=}")

        if not safe_float(inputs[element][0], "") is None:
            inputs[element][0] = unit_dict[inputs[element][3]] * safe_float(inputs[element][0], "")


def solveEquation(formula, inputs):
    # Define symbols for variables based on the keys of the input_dict
    symbols_list = symbols(' '.join(inputs.keys()))

    ls, rs = formula.split("==")

    ls = sympify(ls)
    rs = sympify(rs)

    eqn = Eq(ls, rs)

    for i, element in enumerate(inputs):
        eqn = eqn.subs(symbols_list[i], inputs[element][0])

    sol = sp.solve(eqn)

    return sol


def list_to_5_sig_figs(sol):
    for i, element in enumerate(sol):
        sol[i] = round(float(element), -int(floor(log10(abs(element)))) + 4)

# solve(info)



from TopicFactory import TopicFactory


class ThreeDPlotter(TopicFactory):
    def __init__(self):

        self.input_dict = {
            "function": ["", "sin(sqrt(x**2 + y**2))", "length", "", ""],
        }

        self.info = {
            "input": self.input_dict,
            "formula": "",
            "Note": "This is a 3D Plotter\n"
                    "Write the function so that the\n"
                    "dependant variable is omitted.\n"
                    "i.e. write z = x+y as x+y.",
            "solve_method": "",
            "plot_method": False,
            "Bonus": self.Bonus
        }

    def giveInfo(self):
        return self.info

    def Bonus(self, info):
        return self.selfPlot(info["input"]["function"][0])

    def selfPlot(self, function):
        import numpy as np
        import matplotlib.pyplot as plt
        from mpl_toolkits.mplot3d import Axes3D
        from sympy.parsing.sympy_parser import (parse_expr, standard_transformations,
                                                implicit_multiplication_application)
        # Define the function to plot as a string
        function_str = function

        # Define the range of values for x and y
        x_range = np.linspace(-10, 10, 100)
        y_range = np.linspace(-10, 10, 100)

        # Create a meshgrid of the x and y values
        x_mesh, y_mesh = np.meshgrid(x_range, y_range)

        # Parse the function string into a SymPy expression
        transformations = (standard_transformations + (implicit_multiplication_application,))
        function_expr = parse_expr(function_str, transformations=transformations)

        # Convert the SymPy expression to a NumPy function that can be evaluated on arrays
        f = np.vectorize(lambda x, y: function_expr.evalf(subs={'x': x, 'y': y}))

        # Evaluate the function for the x and y values
        z_mesh = f(x_mesh, y_mesh)

        # Create the figure and the 3D axes
        fig = plt.figure()
        ax = fig.add_subplot(111, projection='3d')

        # Plot the surface of the function
        ax.plot_surface(x_mesh, y_mesh, z_mesh)

        # Set labels for the axes
        ax.set_xlabel('X Label')
        ax.set_ylabel('Y Label')
        ax.set_zlabel('Z Label')

        # Show the plot
        plt.show(block="false")


from UnitFactory import UnitFactory

class angle(UnitFactory):
    def __init__(self):
        self.unit_dict = {
            "radians": 1,
            "degrees": 0.0174533,
        }

    def giveDict(self):
        return self.unit_dict



from UnitFactory import UnitFactory

class length(UnitFactory):
    def __init__(self):
        self.unit_dict = {
            "m": 1,
            "km": 1000,
            "mm": .001,
            "ft": 0.3048,
            "in": 0.0254,
            "yard": 0.9144,
        }

    def giveDict(self):
        return self.unit_dict



from src.TopicFactory import TopicFactory

from math import sin

class Projectile_Distance_in_x(TopicFactory):
    def __init__(self):

        g = 9.81

        self.input_dict = {
            "xtot": ["", 3, "length", "", ""],
            "vi": ["", 4, "velocity", "", ""],
            "time": ["", 5, "time", "", ""],
            "theta": ["", 3, "angle", "", ""],
                      }

        self.info = {
             "input": self.input_dict,
             "formula": f"xtot == (vi ** 2)/{g} * sin(2 * theta) ",
             "Note": """
                    This is the total distance travelled by the projectile in the x direction as a function 
                    of velocity, and angle. This is a special case where the starting and ending height must be
                    the same. 
                    """,
             "solve_method": "",
             "plot_method": False,
             "Bonus": self.Bonus
             }

    def giveInfo(self):
        return self.info


import math
import matplotlib.pyplot as plt
from TopicFactory import TopicFactory

# Define the input stresses

class mohrsCircle(TopicFactory):
    def __init__(self):
        self.input_dict = {
            "sigmaX": ["", "", "length", "", ""],
            "sigmaY": ["", "", "length", "", ""],
            "tauXY": ["", "", "length", "", ""],
                      }

        self.info = {
             "input": self.input_dict,
             "formula": "",
             "Note": "This is Mohr's Circle",
             "solve_method": "",
             "plot_method": False,
             "Bonus": self.Bonus
             }

    def giveInfo(self):
        return self.info

    def Bonus(self, info):
        return self.selfSolve(info["input"]["sigmaX"][0], info["input"]["sigmaY"][0], info["input"]["tauXY"][0])
    def selfSolve(self, sigma_x, sigma_y, tau_xy):
        Center = (sigma_x + sigma_y)/2
        Radius = math.sqrt(((sigma_x-sigma_y)/2)**2 + tau_xy**2)
        calc1 = Center+Radius
        calc2 = Center-Radius
        case1 = False
        case2 = False

        if (calc1 > 0) & (calc2 <= 0):
            case1 = True
            sigma1 = calc1
            sigma2 = 0
            sigma3 = calc2
        elif (calc1 > 0) & (calc2 > 0):
            case2 = True
            sigma1 = calc1
            sigma2 = calc2
            sigma3 = 0
        else:
            sigma1 = 0
            sigma2 = calc1
            sigma3 = calc2


        tauIn = Radius
        tauAll = abs((sigma1-sigma3)/2)

        if (sigma_x - sigma_y) == 0:
            thetaP = 45
        else:
            thetaP = math.degrees((1/2) * math.atan((2 * tau_xy) / (sigma_x - sigma_y)))

        if tau_xy == 0:
            thetaS = -45
        else:
            thetaS = math.degrees(-(1/2) * math.atan((sigma_x - sigma_y) / (2 * tau_xy)))


        if thetaP > 0:
            thetaP90 = thetaP-90
        else:
            thetaP90 = thetaP+90



        if thetaS > 0:
            thetaS90 = thetaS-90
        else:
            thetaS90 = thetaS+90


        sigmaThetaP = ((sigma_x + sigma_y) / 2) + ((sigma_x - sigma_y) / 2) * math.cos(math.radians(2 * thetaP)) + tau_xy * math.sin(math.degrees(2 * thetaP))
        tauXYThetaS = -((sigma_x - sigma_y) / 2) * math.sin(math.radians(2 * thetaS)) + tau_xy * math.cos(math.degrees(2 * thetaS))
        sigmaThetaP90 = ((sigma_x + sigma_y) / 2) + ((sigma_x - sigma_y) / 2) * math.cos(math.degrees(2 * thetaP90)) + tau_xy * math.sin(math.degrees(2 * thetaP90))


        if sigmaThetaP < sigmaThetaP90:
            thetaP = thetaP90
        if tauXYThetaS < tauIn:
            thetaS = thetaS90


        # Calculate the center and radius of the circles for each plane
        center_xy = [(sigma_x + sigma_y)/2, 0]
        center_xz = [(sigma_x - sigma_y)/2, 0]
        center_yz = [(sigma_y - sigma_x)/2, 0]
        R_xy = math.sqrt((sigma_x - center_xy[0])**2 + tau_xy**2)
        R_xz = math.sqrt(center_xz[0]**2 + tau_xy**2)
        R_yz = math.sqrt(center_yz[0]**2 + tau_xy**2)

        # Define the coordinates of the circles
        theta_c = [i*math.pi/180 for i in range(0, 360)]
        fig, ax1 = plt.subplots()
        ax1.invert_yaxis()


        ax1.axhline(0, color='black', linewidth=1)
        ax1.axvline(0, color='black', linewidth=1)



        if case1:
            xunit = [((sigma1 + abs(sigma3)) / 2) * math.cos(t) + ((sigma1 + sigma3) / 2) for t in theta_c]
            yunit = [((sigma1 + abs(sigma3)) / 2) * math.sin(t) for t in theta_c]
            ax1.plot(xunit, yunit,color='black', linewidth=1.5)

            xunit = [((sigma2+sigma3)/2) * math.cos(t) + ((sigma2+sigma3)/2) for t in theta_c]
            yunit = [((sigma2+sigma3)/2) * math.sin(t)for t in theta_c]
            ax1.plot(xunit, yunit,color='black', linewidth=1.5)

            xunit = [((sigma1+sigma2)/2) * math.cos(t) + ((sigma1+sigma2)/2) for t in theta_c]
            yunit = [((sigma1+sigma2)/2) * math.sin(t) for t in theta_c]
            ax1.plot(xunit, yunit,color='black', linewidth=1.5)
        elif case2:
            xunit = [((sigma1+sigma3)/2) * math.cos(t) + ((sigma1+sigma3)/2) for t in theta_c]
            yunit = [((sigma1+sigma3)/2) * math.sin(t) for t in theta_c]
            ax1.plot(xunit, yunit,color='black', linewidth=1.5)

            xunit = [((sigma2+sigma3)/2) * math.cos(t) + ((sigma2+sigma3)/2) for t in theta_c]
            yunit = [((sigma2+sigma3)/2) * math.sin(t) for t in theta_c]
            ax1.plot(xunit, yunit,color='black', linewidth=1.5)

            xunit = [((sigma1-sigma2)/2) * math.cos(t) + ((sigma1+sigma2)/2) for t in theta_c]
            yunit = [((sigma1-sigma2)/2) * math.sin(t) for t in theta_c]
            ax1.plot(xunit, yunit,color='black', linewidth=1.5)
        else:
            xunit = [((sigma1+sigma3)/2) * math.cos(t) + ((sigma1+sigma3)/2) for t in theta_c]
            yunit = [((sigma1+sigma3)/2) * math.sin(t) for t in theta_c]
            ax1.plot(xunit, yunit,color='black', linewidth=1.5)

            xunit = [((sigma2-sigma3)/2) * math.cos(t) + ((sigma2+sigma3)/2) for t in theta_c]
            yunit = [((sigma2-sigma3)/2) * math.sin(t) for t in theta_c]
            ax1.plot(xunit, yunit,color='black', linewidth=1.5)

            xunit = [((sigma1-sigma2)/2) * math.cos(t) + ((sigma1+sigma2)/2)for t in theta_c]
            yunit = [((sigma1-sigma2)/2) * math.sin(t) for t in theta_c]
            ax1.plot(xunit, yunit,color='black', linewidth=1.5)

        #dots the
        if abs(tauAll-tauIn) > abs(tauIn/100000):
            ax1.scatter(Center,tauIn, color='black', marker='o', s=50)
            ax1.annotate(r"$\tau_{in}$", xy=(Center, (Radius+(Radius/10))))
            ax1.scatter(((sigma1+sigma3)/2),tauAll, color='black', marker='o', s=50)
            ax1.annotate(r"$\tau_{all}$", xy=(((sigma1+sigma3)/2), (tauAll+(Radius/10))))
        else:
            ax1.scatter(Center, tauIn, color='black', marker='o', s=50)
            ax1.annotate(r"$\tau_{in&all}$", xy=(Center, (Radius+(Radius/10))))



        #display dotted angle of theta_p
        upperBound = -2*thetaP
        if upperBound < 0:
            if sigma_x>sigma_y:
                th = [i*math.pi/180 for i in range(0,int(-upperBound))]
                ax1.annotate(r"$\theta_p$", xy=(((Radius/5)+Center),(Radius/10)))
            else:
                th = [i * math.pi / 180 for i in range(-int(180+upperBound),0)]
                ax1.annotate(r"$\theta_p$", xy=(((Radius / 5) + Center), -(Radius / 10)))
        else:
            if sigma_x > sigma_y:
                th = [i * math.pi / 180 for i in range(int(-upperBound),0)]
                ax1.annotate(r"$\theta_p$", xy=(((Radius/5)+Center),(-Radius/10)))
            else:
                th = [i * math.pi / 180 for i in range(0, int(180+upperBound))]
                ax1.annotate(r"$\theta_p$", xy=(((Radius / 5) + Center), (Radius / 10)))
        xunit = [(Radius/5) * math.cos(t) + Center for t in th]
        yunit = [(Radius/5) * math.sin(t) for t in th]
        ax1.plot(xunit, yunit, linestyle='--',color='black', linewidth=1)
        print(upperBound)

        #display dotted angle of theta_s
        if upperBound < 0:
            if sigma_x>sigma_y:
                th = [i*math.pi/180 for i in range( int(-upperBound),90)]
                ax1.annotate(r"$\theta_s$", xy=(Center,tauIn/2.5))
            else:
                th = [i * math.pi / 180 for i in range(90,-int(upperBound-180))]
                ax1.annotate(r"$\theta_s$", xy=(Center, tauIn / 2.5))
        else:
            if sigma_x>sigma_y:
                th = [i * math.pi / 180 for i in range(int((-upperBound)),90)]
                ax1.annotate(r"$\theta_s$", xy=(Center,tauIn/2.5))
            else:
                th = [i * math.pi / 180 for i in range(int(90 + (upperBound)), 270)]
                ax1.annotate(r"$\theta_s$", xy=(Center, -tauIn / 3))

        print(upperBound)
        xunit = [(Radius/3) * math.cos(t) + Center for t in th]
        yunit = [(Radius/3) * math.sin(t) for t in th]
        ax1.plot(xunit, yunit, linestyle='--',color='black', linewidth=1)



        ax1.plot([Center,Center], [0,Radius], linestyle='--', linewidth=1, color="black")
        ax1.scatter(Center,0, color='black', marker='o', s=50)
        ax1.scatter(sigma1,0, color='black', marker='o', s=50)
        ax1.annotate(r"$\sigma_1$", xy=(sigma1,(Radius/10)))
        ax1.scatter(sigma2,0, color='black', marker='o', s=50)
        ax1.annotate(r"$\sigma_2$", xy=(sigma2,(Radius/10)))
        ax1.scatter(sigma3,0, color='black', marker='o', s=50)
        ax1.annotate(r"$\sigma_3$", xy=(sigma3,(Radius/10)))
        ax1.scatter(sigma_x,tau_xy, color='black', marker='o', s=50)
        ax1.annotate(r"$X$", xy=(sigma_x,tau_xy))
        ax1.annotate(r"$Y$", xy=(sigma_y,-tau_xy))
        ax1.scatter(sigma_y,-tau_xy, color='black', marker='o', s=50)
        # Plot the line on all planes where sigma_x = sigma_y
        ax1.plot([sigma_x, sigma_y], [tau_xy, -tau_xy], linestyle='--', color='black',linewidth=1)

        # Set the aspect ratio and axis labels
        ax1.set_aspect('equal', 'box')
        ax1.set_xlabel('Normal stress (MPa)')
        ax1.set_ylabel('Shear stress (MPa)')
        output = {
            "sigma1": [sigma1, "", "length", "", ""],
            "sigma2": [sigma2, "", "length", "", ""],
            "sigma3": [sigma3, "", "length", "", ""],
            "tauIn": [tauIn, "", "length", "", ""],
            "tauAll": [tauAll, "", "length", "", ""],
            "thetaP": [thetaP, "", "length", "", ""],
            "thetaS": [thetaS, "", "length", "", ""],
                      }

        # Add the x and y axes to each plot


        # tau_max_xy = R_xy
        # print(f"Plane XY - Principal stresses: sigma_1 = {sigma_1_xy:.2f} MPa, sigma_2 = {sigma_2_xy:.2f} MPa")
        # print(f"Plane XY - Maximum shear stress: tau_max = {tau_max_xy:.2f} MPa")
        # sigma_1_xz = center_xz[0] + R_xz
        # sigma_2_xz = -sigma_1_xz
        # tau_max_xz = R_xz
        # print(f"Plane XZ - Principal stresses: sigma_1 = {sigma_1_xz:.2f} MPa, sigma_2 = {sigma_2_xz:.2f} MPa")
        # print(f"Plane XZ - Maximum shear stress: tau_max = {tau_max_xz:.2f} MPa")
        # sigma_1_yz = center_yz[0] + R_yz
        # sigma_2_yz = -sigma_1_yz
        # tau_max_yz = R_yz
        # print(f"Plane YZ - Principal stresses: sigma_1 = {sigma_1_yz:.2f} MPa, sigma_2 = {sigma_2_yz:.2f} MPa")
        # print(f"Plane YZ - Maximum shear stress: tau_max = {tau_max_yz:.2f} MPa")
        #
        # # Label the point on the XY plane where the maximum shear stress occurs
        # ax1.annotate(f"({center_xy[0]:.2f}, {tau_max_xy:.2f})", xy=(center_xy[0], tau_max_xy), xytext=(center_xy[0]-1, tau_max_xy+1),
        #             arrowprops=dict(facecolor='black', arrowstyle='->'))


        # Display the plot

        plt.grid(linestyle='--', color='grey', linewidth=0.5)
        plt.show(block='False')
        return output


from src.TopicFactory import TopicFactory

from math import cos

class Initial_Velocity_in_x(TopicFactory):
    def __init__(self):

        g = 9.81

        self.input_dict = {
            "vxi": ["", 3, "velocity", "", ""],
            "vi": ["", 4, "velocity", "", ""],
            "time": ["", 5, "time", "", ""],
            "theta": ["", 3, "angle", "", ""],
                      }

        self.info = {
             "input": self.input_dict,
             "formula": f"vxi == vi * cos(theta)",
             "Note": """
                    This is the initial velocity in x as a function of the initial velocity and the angle of 
                    projection
                    """,
             "solve_method": "",
             "plot_method": False,
             "Bonus": self.Bonus
             }

    def giveInfo(self):
        return self.info


from UnitFactory import UnitFactory

class stress(UnitFactory):
    def __init__(self):
        self.unit_dict = {
            "kPa": 1,
            "Pa": 0.001,
            "psi": 6.89476,
        }

    def giveDict(self):
        return self.unit_dict



from UnitFactory import UnitFactory

class mass(UnitFactory):
    def __init__(self):
        self.unit_dict = {
            "kg": 1,
            "g": .001,
            "Mg": 1000,
            "lb": 2.2046,
            "kip": 2.2046 * 1088-3,
            "oz": 0.0283495,
            "metric ton": 1000,
            "ozt": 0.0311035,
        }

    def giveDict(self):
        return self.unit_dict



from src.TopicFactory import TopicFactory


class Angular_Acceleration(TopicFactory):
    def __init__(self):

        g = 9.81

        self.input_dict = {
            "radialposition": ["", 3, "angle", "", ""],
            "omegai": ["", 4, "velocity", "", ""],
            "time": ["", 5, "time", "", ""],
            "omegadot" : ["", 3, "acceleration", "", ""],
            "radialpositioni": ["", 2, "angle", "", ""],
            "omegadoti" : ["", 3, "acceleration", "", ""]
                      }

        self.info = {
             "input": self.input_dict,
             "formula": f"omegadot ** 2 == omegadoti ** 2 + 2 * omegadot * (radialposition - radialpositioni)",
             "Note": """
                     This is the angular acceleration of an object as a function of angular acceleration and 
                     angular velocity.
                     """,
             "solve_method": "",
             "plot_method": False,
             "Bonus": self.Bonus
             }

    def giveInfo(self):
        return self.info


from usu_apex.UnitFactory import UnitFactory

class pressure(UnitFactory):
    def __init__(self):
        self.unit_dict = {
            "kPa": 1,
            "Pa": 0.001,
            "psi": 6.89476,
            "bar": 100,
            "atm": 101.325,
            "mmHg": 0.133322,
        }

    def giveDict(self):
        return self.unit_dict



from UnitFactory import UnitFactory

class torque(UnitFactory):
    def __init__(self):
        self.unit_dict = {
            "Nm": 1,
            "ftlb": 0.7376,
            "inlb": 8.851,
            "mNm": 1000,
            "gf cm": 10197,
        }

    def giveDict(self):
        return self.unit_dict



from TopicFactory import TopicFactory


class ThreeDPlotter(TopicFactory):
    def __init__(self):

        self.input_dict = {
            "function": ["", "sin(x) + cos(x)", "length", "", ""],
            "x max": ["", "10", "length", ""],
            "x min": ["", "0", "length", "", ""],
        }

        self.info = {
            "input": self.input_dict,
            "formula": "",
            "Note": "This is a 2D Plotter",
            "solve_method": "",
            "plot_method": False,
            "Bonus": self.Bonus
        }

    def giveInfo(self):
        return self.info

    def Bonus(self, info):
        return self.selfPlot(info["input"]["function"][0], info["input"]["x max"][0], info["input"]["x min"][0])
    def selfPlot(self, function, xMax, xMin):
        import numpy as np
        import matplotlib.pyplot as plt
        from sympy.parsing.sympy_parser import (parse_expr, standard_transformations,
                                                implicit_multiplication_application)
        function_str = function

        # Define the range of values for x
        x_range = np.linspace(xMin, xMax, 100)

        # Parse the function string into a SymPy expression
        transformations = (standard_transformations + (implicit_multiplication_application,))
        function_expr = parse_expr(function_str, transformations=transformations)

        # Convert the SymPy expression to a NumPy function that can be evaluated on arrays
        f = np.vectorize(lambda x: function_expr.evalf(subs={'x': x}))

        # Evaluate the function for the x values
        y_values = f(x_range)

        # Find the max, min, and x-intercept of the function
        max_y = max(y_values)
        min_y = min(y_values)
        x_intercept = x_range[np.where(np.diff(np.sign(y_values)))[0][0]]

        # Create the plot and add the function curve
        fig, ax = plt.subplots()
        ax.plot(x_range, y_values)
        ax.axhline(0, color='black')
        ax.axvline(0, color='black')
        # Annotate the max, min, and x-intercept of the function on the plot

        # Set labels for the axes and the title of the plot
        ax.set_xlabel('X')
        ax.set_ylabel('Y')
        ax.set_title(f"Plot of {function_str}")

        # Show the plot
        plt.show()


from UnitFactory import UnitFactory

class force(UnitFactory):
    def __init__(self):
        self.unit_dict = {
            "kN": 1,
            "MN": 1000,
            "N": .001,
            "lb": 224.8089431,
            "kip": .2248089431,
        }

    def giveDict(self):
        return self.unit_dict



from UnitFactory import UnitFactory

class velocity(UnitFactory):
    def __init__(self):
        self.unit_dict = {
            "m/s": 1,
            "cm/s": 100,
            "ft/s": 0.3048,
            "ft/min": 0.00508,
            "km/h": 0.277778,
            "mph": 0.44704,
            "knot": 0.514444444,
        }

    def giveDict(self):
        return self.unit_dict



from src.TopicFactory import TopicFactory


class Distnace_in_y(TopicFactory):
    def __init__(self):

        g = 9.81

        self.input_dict = {
            "y": ["", 3, "length", "", ""],
            "yi": ["", 4, "length", "", ""],
            "time": ["", 5, "time", "", ""],
            "vyi" : ["", 3, "velocity", "", ""],
                      }

        self.info = {
             "input": self.input_dict,
             "formula": f"y == yi + vyi * time - .5 * {g} * time**2",
             "Note": "This is the velocity of the projectile at any time",
             "solve_method": "",
             "plot_method": False,
             "Bonus": self.Bonus
             }

    def giveInfo(self):
        return self.info


from src.TopicFactory import TopicFactory
import numpy as np

class Orbital_Period(TopicFactory):
    def __init__(self):

        G = 6.6743 * np.power(10, -11)

        self.input_dict = {
            "orbitalradius": ["", 3, "length", "", ""],
            "massorbitant": ["", 4, "mass", "", ""],
            "time": ["", 5, "time", "", ""],
            "orbitalperiod": ["", 3, "time", "", ""],
                      }

        self.info = {
             "input": self.input_dict,
             "formula": f"orbitalperiod == np.sqrt((4 * np.pi**2 * np.power(orbitalradius, 3) / {G} * massorbitant)",
             "Note": """
                     This is the orbital period of an object around a celestial body as a function of radius and mass
                     """,
             "solve_method": "",
             "plot_method": False,
             "Bonus": self.Bonus
             }

    def giveInfo(self):
        return self.info


from usu_apex.TopicFactory import TopicFactory


class Law_of_Cosines(TopicFactory):
    def __init__(self):
        self.input_dict = {
            "a": ["", 14.62, "length", "", ""],
            "b": ["", 9, "length", "", ""],
            "c": ["", 12, "length", "", ""],
            "A": ["", "", "angle", "", ""]
        }

        self.info = {
             "input": self.input_dict,
             "formula": "a**2 == b**2 + c**2 - 2 * b * c * cos(A)",
             "Note": "This is the law of cosines",
             "solve_method": "",
             "plot_method": False,
             "Bonus": self.Bonus
             }

    def giveInfo(self):
        return self.info



from src.TopicFactory import TopicFactory

from math import cos

class Projectile_Distance_in_x(TopicFactory):
    def __init__(self):



        self.input_dict = {
            "xtot": ["", 3, "length", "", ""],
            "vi": ["", 4, "velocity", "", ""],
            "time": ["", 5, "time", "", ""],
            "theta": ["", 3, "angle", "", ""],
                      }

        self.info = {
             "input": self.input_dict,
             "formula": "xtot == vi * cos(theta) * time",
             "Note": """
                    This is the total distance travelled by the projectile in 
                    the x direction as a function of velocity, angle, and time
                    """,
             "solve_method": "",
             "plot_method": False,
             "Bonus": self.Bonus
             }

    def giveInfo(self):
        return self.info


import numpy as np
import sympy
x, y2, y1 = sympy.symbols('x y2 y1')

# Example input list with string numbers
output_dict = ["17 -38*np.cos(np.pi/4)", "-10 30 38*np.cos(np.pi/4)", "-10 90 (5*38*np.cos(np.pi/4))"]

variables_passed = [[0], [y1, y2, -10], [-20, 4]]
"""""
for string in output_dict:
    split_string = string.split()
    new_sublist = []
    for substring in split_string:
        if '*' in substring:  # check if the substring contains multiplication
            new_sublist.append(eval(substring))  # evaluate the expression and append to the new sublist
        else:
            new_sublist.append(float(substring))  # convert the string to float and append to the new sublist
    variables_passed.append(new_sublist)

print(variables_passed)
"""
part1 = []
part2 = []
for i in range(3):
    if i == 1:
        continue
    part1.append(variables_passed[i])



variables = [x, y2]


part1[0].extend([variables[0]])
part1[1][1]= part1[1][1] * y2




# Define the equations
eq1 = sympy.Eq(sum(part1[0]), 0)
eq2 = sympy.Eq(sum(part1[1]), 0)


# Solve the system of equations
sol = sympy.solve((eq1, eq2), (x, y2))

# Print the solution

y2_ans = sol[y2]

eq3 = sympy.Eq(sum(variables_passed[1]), 0)

sol2 = sympy.solve((eq3.subs(y2, y2_ans), y2), y1)


sol.update(sol2)
print(sol)


from UnitFactory import UnitFactory

class time(UnitFactory):
    def __init__(self):
        self.unit_dict = {
            "sec": 1,
            "min": 60,
            "hour": 60*60,
            "day": 60*60*24,
            "week": 60*60*24*7,
            "year": 3.154e+7,
            "ms": 1000,
            "μs": 10**6,
            "ns": 10**9,
        }

    def giveDict(self):
        return self.unit_dict



from src.TopicFactory import TopicFactory


class Displacement_in_x(TopicFactory):
    def __init__(self):


        self.input_dict = {
            "x": ["", 3, "length", "", ""],
            "xi": ["", 4, "length", "", ""],
            "time": ["", 5, "time", "", ""],
            "vxi": ["", 2, "velocity", "", ""],
                      }

        self.info = {
             "input": self.input_dict,
             "formula": "x = xi + vxi * time",
             "Note": """
                     This is the distance that a projectile travels in the x direction as a function of initial
                     velocity, time and the initial position in x. 
                     """,
             "solve_method": "",
             "plot_method": False,
             "Bonus": self.Bonus
             }

    def giveInfo(self):
        return self.info


import os
import tkinter as tk
from tkinter import ttk
import Solver

# All the callback functions here

# The current object to be passed around
current_object = None


# Function for when a course is selected from the course drop down box
def courseSelect(event):
    sub_box.set("Subject")
    topic_box.set("Topic")
    course_txt = course_box.get()
    file_list = os.listdir(f"usu_apex/Input_Files/{course_txt}")
    sub_box['values'] = file_list


# Function for when a subject is selected
def subSelect(event):
    topic_box.set("Topic")
    course_txt = course_box.get()
    sub_txt = sub_box.get()
    file_list = os.listdir(f"usu_apex/Input_Files/{course_txt}/{sub_txt}")
    good_files = []
    for i in file_list:
        if i[-3:] == ".py":
            good_files.append(i[:-3])
    topic_box['values'] = good_files


# Function for when a topic is selected
def topicSelect(event):
    # Erase the current inputs
    note_text.delete("1.0", tk.END)
    note_text.insert("1.0", "Note:\n")
    for child in input_frame.winfo_children():
        child.destroy()

    # Build the file path
    course_txt = course_box.get()
    sub_txt = sub_box.get()
    topic_txt = topic_box.get()
    mod_name = f"Input_Files.{course_txt}.{sub_txt}.{topic_txt}"

    # Import the specified module and get make an object from the class inside
    module = __import__(mod_name, fromlist=['*'])

    class_count = 0
    for name, obj in module.__dict__.items():
        if isinstance(obj, type):
            class_count += 1
            if class_count == 2:
                global current_object
                current_object = obj()
                break

    info = current_object.giveInfo()

    # Create the info boxes in the GUI
    note = info['Note']
    note_text.delete("1.0", tk.END)
    note_text.insert("1.0", "Note:\n"+note)

    # Get the units
    itr=0
    for i, element in enumerate(info['input']):
        # print(f"Units.{inputs[element][2]}")
        module = __import__(f"Units.{info['input'][element][2]}", fromlist=['*'])

        class_obj = None
        class_count = 0
        for name, obj in module.__dict__.items():
            if isinstance(obj, type):
                class_count += 1
                if class_count == 2:
                    class_obj = obj
                    break

        my_obj = class_obj()
        unit_dict = my_obj.giveDict()

        label = tk.Label(input_frame, text=element)
        entry = tk.Entry(input_frame)
        entry.insert(0, info['input'][element][1])
        unit = ttk.Combobox(input_frame, values=list(unit_dict.keys()))
        unit.set(list(unit_dict.keys())[0])
        label.grid(row=itr, column=0)
        entry.grid(row=itr, column=1)
        unit.grid(row=itr, column=2)
        itr += 1

    execute.config(state="normal")


# Function for when the execute button is pressed
def execute():
    # Update info with parameters given by the user
    global current_object
    input_children = input_frame.winfo_children()
    info = current_object.giveInfo()
    itr = 0

    for child in input_children:
        if isinstance(child, tk.Label):
            label_txt = child.cget("text")
            info['input'][label_txt][0] = input_children[itr + 1].get()
            info['input'][label_txt][3] = input_children[itr + 2].get()
        itr += 1

    # Call the solver to solve the equation
    output_info = Solver.solve(info)
    #print(output_info)

    # Populate the output
    outputwin = tk.Tk()
    outputwin.geometry("500x500")
    formula = tk.Label(outputwin, text=output_info['formula'])
    formula.grid(row=0)
    inputs = tk.Label(outputwin, text="Input from user:")
    inputs.grid(row=1)

    itr = 2
    for i in info['input']:
        label = tk.Label(outputwin, text=i)
        entry = tk.Label(outputwin, text=str(output_info['input'][i][0]))
        unit = tk.Label(outputwin, text=output_info['input'][i][3])
        label.grid(row=itr, column=0)
        entry.grid(row=itr, column=1)
        unit.grid(row=itr, column=2)
        itr += 1

    outputs = tk.Label(outputwin, text="Output:")
    outputs.grid(row=itr)
    itr += 1
    for i in output_info['output']:
        label = tk.Label(outputwin, text=i)
        entry = tk.Label(outputwin, text=str(output_info['output'][i][0]))
        unit = ttk.Label(outputwin, text=output_info['output'][i][3])
        label.grid(row=itr, column=0)
        entry.grid(row=itr, column=1)
        unit.grid(row=itr, column=2)
        itr += 1


# End of callback functions

# Create root
root = tk.Tk()
root.geometry('500x500')
root.title("Engineering Calculator")

# Initialize frames for organization
top_frame = tk.Frame(root)
top_frame.pack(side='top', fill="both", expand=True)

bottom_frame = tk.Frame(root)
bottom_frame.pack(side='bottom', fill="both", expand=True)

file_frame = tk.Frame(top_frame, relief='groove')
file_frame.pack(side='left', fill="both", expand=True)

note_frame = tk.Frame(top_frame, relief='groove')
note_frame.pack(side="right", fill="both", expand=True)

input_frame = tk.Frame(bottom_frame, relief='groove')
input_frame.pack(fill="both", expand=True)

# create subject drop down boxes
course_box = ttk.Combobox(file_frame)
course_box.set("Course")
sub_box = ttk.Combobox(file_frame)
sub_box.set("Subject")
topic_box = ttk.Combobox(file_frame)
topic_box.set("Topic")

# Populate the course drop down
course_files = os.listdir("Input_Files")
course_box['values'] = course_files

# Bind callback functions for the drop-downs
course_box.bind("<<ComboboxSelected>>", courseSelect)
sub_box.bind("<<ComboboxSelected>>", subSelect)
topic_box.bind("<<ComboboxSelected>>", topicSelect)

# Pack all the drop-downs in tight
course_box.grid(row=0, column=0)
sub_box.grid(row=1, column=0)
topic_box.grid(row=2, column=0)

# Create the note and input label and go button
note_text = tk.Text(note_frame)
note_text.insert(tk.END, "Note:\n")
note_text.grid(row=0)

input_label = tk.Label(input_frame, text="Inputs")
input_label.grid(row=0)

execute = tk.Button(bottom_frame, text="Execute", command=execute, state="disabled")
execute.pack(side="bottom")

# run the mainloop to initialize the GUI
tk.mainloop()



from src.TopicFactory import TopicFactory
import numpy as np

class Escape_Velocity(TopicFactory):
    def __init__(self):

        G = 6.6743 * np.power(10, -11)

        self.input_dict = {
            "vesc": ["", 3, "velocity", "", ""],
            "massorbitant": ["", 4, "mass", "", ""],
            "time": ["", 5, "time", "", ""],
            "radius": ["", 3, "length", "", ""],
                      }

        self.info = {
             "input": self.input_dict,
             "formula": f"vesc == np.sqrt((2 * {G} * massorbitant) / radius)",
             "Note": """
                     This is the angular position as a function of initial position, angular velocity, 
                     and angular acceleration.
                     """,
             "solve_method": "",
             "plot_method": False,
             "Bonus": self.Bonus
             }

    def giveInfo(self):
        return self.info


import numpy as np
import sympy
from TopicFactory import TopicFactory
class Cantilever(TopicFactory):
    def __init__(self):
        self.input_dict = {
            "Forcex": ["", "", "force", "", ""],
            "Forcey": ["", "", "force", "", ""],
            "Moment": ["", "", "torque", "", ""],
        }

        self.info = {
            "input": self.input_dict,
            "formula": "",
            "Note": """
                    This is a cantilerver beam solver. Input all of the x force components into the text box separated by a 
                    space. All moments are taken around the reaction force location at the wall. Input the moment components
                    into the text box in the same way as the forces. Force magnitudes multiplied by their respective trig function
                    are acceptable inputs.
                    """,
            "solve_method": "",
            "plot_method": False,
            "Bonus": self.Bonus
        }

    def giveInfo(self):
            return self.info

    def Bonus(self, info):
        print(info)
        return self.selfSolve(info["input"]["Forcex"][0], info["input"]["Forcey"][0], info["input"]["Moment"][0])

    def selfSolve(self, Forcex, Forcey, Moment):
        # Example input list with string numbers
        output_dict = [Forcex, Forcey, Moment]

        variables_passed = []

        for string in output_dict:
            split_string = string.split()
            new_sublist = []
            for substring in split_string:
                if '*' in substring:  # check if the substring contains multiplication
                    new_sublist.append(eval(substring))  # evaluate the expression and append to the new sublist
                else:
                    new_sublist.append(float(substring))  # convert the string to float and append to the new sublist
            variables_passed.append(new_sublist)

        print(variables_passed)



        x, y, m = sympy.symbols('x y m')

        variables=[x,y,m]

        for i in range(len(variables_passed)):
            variables_passed[i].extend([variables[i]])

        print(variables_passed)

        # Define the equations
        eq1 = sympy.Eq(sum(variables_passed[0]), 0)
        eq2 = sympy.Eq(sum(variables_passed[1]), 0)
        eq3 = sympy.Eq(sum(variables_passed[2]), 0)

        # Solve the system of equations
        sol = sympy.solve((eq1, eq2, eq3), (x, y, m))

        # Print the solution
        print(sol)

        return sol


from usu_apex.UnitFactory import UnitFactory

class acceleration(UnitFactory):
    def __init__(self):
        self.unit_dict = {
            "m/(s)^2": 1,
            "cm/(s)^2": 100,
            "in/(s)^2": 39.37,
            "ft/(s)^2": 3.28084,
            "g": 9.80665
        }

    def giveDict(self):
        return self.unit_dict



from TopicFactory import TopicFactory

class stress(TopicFactory):
    def __init__(self):
        self.input_dict = {
            "F": ["", 3, "length", "", ""],
            "A": ["", 4, "length", "", ""],
            "stress": ["", "", "length", "", ""],
                      }

        self.info = {
             "input": self.input_dict,
             "formula": "stress == F/A",
             "Note": "This is a stress calculator",
             "solve_method": "",
             "plot_method": False,
             "Bonus": self.Bonus
             }

    def giveInfo(self):
        return self.info



from src.TopicFactory import TopicFactory


class Velocity_in_y(TopicFactory):
    def __init__(self):

        g = 9.81

        self.input_dict = {
            "vy": ["", 3, "velocity", "", ""],
            "vyi": ["", 4, "velocity", "", ""],
            "time": ["", 5, "time", "", ""],
                      }

        self.info = {
             "input": self.input_dict,
             "formula": f"vy == vyi - {g} * time",
             "Note": "This is the velocity of the projectile at any time",
             "solve_method": "",
             "plot_method": False,
             "Bonus": self.Bonus
             }

    def giveInfo(self):
        return self.info


from src.TopicFactory import TopicFactory

from math import sin

class Projectile_Distance_in_x(TopicFactory):
    def __init__(self):

        g = 9.81

        self.input_dict = {
            "ytot": ["", 3, "length", "", ""],
            "vi": ["", 4, "velocity", "", ""],
            "time": ["", 5, "time", "", ""],
            "theta": ["", 3, "angle", "", ""],
                      }

        self.info = {
             "input": self.input_dict,
             "formula": f"ytot == vi * sin(theta) * time -.5 * {g} * time ** 2",
             "Note": """
                    This is the total distance travelled by the projectile in the y direction as a function 
                    of velocity, angle, and time
                    """,
             "solve_method": "",
             "plot_method": False,
             "Bonus": self.Bonus
             }

    def giveInfo(self):
        return self.info